# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.core.management.base import BaseCommand

from djangocms_modules.models import Category


class Command(BaseCommand):
    help = 'Updates the language for each plugin in a module category'

    def handle(self, *args, **options):
        count = 0
        language = settings.LANGUAGE_CODE
        categories = Category.objects.select_related('modules')

        for category in categories.iterator():
            count += 1
            category.modules.cmsplugin_set.update(language=language)

        self.stdout.write('Successfully updated "%d" module categories.' % count)



# -*- coding: utf-8 -*-
import json

from django.apps import apps
from django.urls import Resolver404, resolve

from cms import operations
from cms.models import CMSPlugin

from .models import Category, ModulePlugin


def sync_module_category(sender, **kwargs):
    from djangocms_history.actions import (
        MOVE_IN_PLUGIN,
        MOVE_OUT_PLUGIN,
    )

    operation = kwargs['operation']
    actions = kwargs['actions']
    affected_operations = (operations.MOVE_PLUGIN, operations.PASTE_PLUGIN)

    if operation.operation_type not in affected_operations:
        return

    try:
        match = resolve(operation.origin)
    except Resolver404:
        match = None

    is_in_modules = match and match.url_name == 'cms_modules_list'

    if not is_in_modules:
        return

    if operation.operation_type == operations.PASTE_PLUGIN:
        if not operation.is_applied:
            # Nothing to do because undoing a paste deletes the plugin.
            return
        # User is redoing a paste
        action = actions[0]
        action_data = json.loads(action.post_action_data)
    elif operation.is_applied:
        # User is redoing moving a plugin out of a placeholder
        action = [action for action in actions if action.action == MOVE_IN_PLUGIN][0]
        action_data = json.loads(action.post_action_data)
    else:
        # User is undoing moving a plugin out of a placeholder
        action = [action for action in actions if action.action == MOVE_OUT_PLUGIN][0]
        action_data = json.loads(action.pre_action_data)

    placeholder = action.placeholder
    first_plugin = action_data['plugins'][0]

    if first_plugin['plugin_type'] == 'Module':
        new_category = Category.objects.get(modules=placeholder)
        root_plugin = CMSPlugin.objects.get(pk=first_plugin['pk'])
        (ModulePlugin
         .objects
         .filter(path__startswith=root_plugin.path, depth__gte=root_plugin.depth)
         .update(module_category=new_category))


if apps.is_installed('djangocms_history'):
    from djangocms_history import signals

    signals.post_operation_undo.connect(
        sync_module_category,
        dispatch_uid='undo_sync_module_category',
    )

    signals.post_operation_redo.connect(
        sync_module_category,
        dispatch_uid='redo_sync_module_category',
    )



# -*- coding: utf-8 -*-
# Generated by Django 2.1.5 on 2019-01-22 04:12
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangocms_modules', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='category',
            options={'verbose_name': 'Category', 'verbose_name_plural': 'Categories'},
        ),
        migrations.AlterField(
            model_name='category',
            name='name',
            field=models.CharField(max_length=120, unique=True, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='moduleplugin',
            name='module_category',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djangocms_modules.Category', verbose_name='Category'),
        ),
        migrations.AlterField(
            model_name='moduleplugin',
            name='module_name',
            field=models.CharField(max_length=120, verbose_name='Name'),
        ),
    ]



# -*- coding: utf-8 -*-
import copy
import json

from django.conf import settings
from django.conf.urls import url
from django.core.exceptions import PermissionDenied
from django.http import (
    HttpResponse, HttpResponseBadRequest, HttpResponseForbidden,
)
from django.shortcuts import get_object_or_404, render
from django.urls import reverse
from django.utils.encoding import force_text
from django.utils.http import urlencode
from django.utils.translation import get_language_from_request
from django.utils.translation import ugettext_lazy as _
from django.views.generic import ListView

from cms import operations
from cms.exceptions import PluginLimitReached
from cms.models import CMSPlugin
from cms.plugin_base import CMSPluginBase, PluginMenuItem
from cms.plugin_pool import plugin_pool
from cms.utils.plugins import (
    copy_plugins_to_placeholder, get_bound_plugins, has_reached_plugin_limit,
    reorder_plugins,
)
from cms.utils.urlutils import admin_reverse

from .forms import AddModuleForm, CreateModuleForm, NewModuleForm
from .models import Category, ModulePlugin


def post_add_plugin(operation, **kwargs):
    from djangocms_history.actions import ADD_PLUGIN
    from djangocms_history.helpers import get_plugin_data
    from djangocms_history.models import dump_json

    module_plugin = kwargs['plugin']
    descendants = module_plugin.get_descendants()
    descendants_bound = get_bound_plugins(descendants)

    # Extend the recorded added plugins to include any nested plugin
    action = operation.actions.only('post_action_data').get(action=ADD_PLUGIN, order=1)
    post_data = json.loads(action.post_action_data)
    post_data['plugins'].extend(get_plugin_data(plugin) for plugin in descendants_bound)
    action.post_action_data = dump_json(post_data)
    action.save(update_fields=['post_action_data'])


class Module(CMSPluginBase):
    name = _('Module')
    allow_children = True
    model = ModulePlugin
    render_template = 'djangocms_modules/render_module.html'
    confirmation_cookie_name = 'modules_disable_confirmation'
    readonly_fields = ['module_category']
    # These are executed by the djangocms-history app
    operation_handler_callbacks = {
        'post_add_plugin': post_add_plugin,
    }

    def has_add_permission(self, request):
        return False

    def get_plugin_urls(self):
        urlpatterns = [
            url(r'^create-module/$', self.create_module_view, name='cms_create_module'),
            url(r'^add-module/(?P<module_id>[0-9]+)/$', self.add_module_view, name='cms_add_module'),
            url(r'^modules/$', self.modules_list_view, name='cms_modules_list'),
        ]
        return urlpatterns

    @classmethod
    def get_extra_plugin_menu_items(cls, request, plugin):
        if plugin.plugin_type == cls.__name__:
            return

        data = {
            'language': get_language_from_request(request, check_path=True),
            'plugin': plugin.pk,
        }
        endpoint = admin_reverse('cms_create_module') + '?' + urlencode(data)
        return [
            PluginMenuItem(
                _('Create module'),
                endpoint,
                action='modal',
                attributes={
                    'icon': 'modules'
                }
            )
        ]

    @classmethod
    def get_extra_placeholder_menu_items(cls, request, placeholder):
        data = {
            'language': get_language_from_request(request, check_path=True),
            'placeholder': placeholder.pk,
        }
        endpoint = admin_reverse('cms_create_module') + '?' + urlencode(data)
        return [
            PluginMenuItem(
                _('Create module'),
                endpoint,
                action='modal',
                attributes={
                    'icon': 'modules'
                }
            )
        ]

    @classmethod
    def create_module_plugin(cls, name, category, plugins):
        placeholder = category.modules
        position = placeholder.get_plugins().filter(parent__isnull=True).count()
        plugin_kwargs = {
            'plugin_type': cls.__name__,
            'placeholder_id': category.modules_id,
            'language': settings.LANGUAGE_CODE,
            'position': position,
        }
        plugin = CMSPlugin.add_root(**plugin_kwargs)
        instance = cls.model(module_name=name, module_category=category)
        plugin.set_base_attr(instance)
        instance.save()
        copy_plugins_to_placeholder(
            plugins,
            placeholder=placeholder,
            language=plugin.language,
            root_plugin=plugin,
        )

    @classmethod
    def create_module_view(cls, request):
        if not request.user.is_staff:
            raise PermissionDenied

        new_form = NewModuleForm(request.GET or None)

        if new_form.is_valid():
            initial_data = new_form.cleaned_data
        else:
            initial_data = None

        if request.method == 'GET' and not new_form.is_valid():
            return HttpResponseBadRequest('Form received unexpected values')

        create_form = CreateModuleForm(request.POST or None, initial=initial_data)
        create_form.set_category_widget(request)

        if not create_form.is_valid():
            opts = cls.model._meta
            context = {
                'form': create_form,
                'has_change_permission': True,
                'opts': opts,
                'root_path': reverse('admin:index'),
                'is_popup': True,
                'app_label': opts.app_label,
                'media': (cls().media + create_form.media),
            }
            return render(request, 'djangocms_modules/create_module.html', context)

        plugins = create_form.get_plugins()

        if not plugins:
            return HttpResponseBadRequest('Plugins are required to create a module')

        name = create_form.cleaned_data['name']
        category = create_form.cleaned_data['category']

        if not category.modules.has_add_plugins_permission(request.user, plugins):
            raise PermissionDenied

        cls.create_module_plugin(name=name, category=category, plugins=plugins)
        return HttpResponse('<div><div class="messagelist"><div class="success"></div></div></div>')

    @classmethod
    def add_module_view(cls, request, module_id):
        if not request.user.is_staff:
            raise PermissionDenied

        module_plugin = get_object_or_404(cls.model, pk=module_id)

        if request.method == 'GET':
            form = AddModuleForm(request.GET)
        else:
            form = AddModuleForm(request.POST)

        if not form.is_valid():
            return HttpResponseBadRequest('Form received unexpected values')

        if request.method == 'GET':
            opts = cls.model._meta
            context = {
                'form': form,
                'has_change_permission': True,
                'opts': opts,
                'root_path': reverse('admin:index'),
                'is_popup': True,
                'app_label': opts.app_label,
                'module': module_plugin,
            }
            return render(request, 'djangocms_modules/add_module.html', context)

        language = form.cleaned_data['target_language']
        target_placeholder = form.cleaned_data.get('target_placeholder')

        if target_placeholder:
            target_plugin = None
        else:
            target_plugin = form.cleaned_data['target_plugin']
            target_placeholder = target_plugin.placeholder

        if not target_placeholder.has_add_plugin_permission(request.user, module_plugin.plugin_type):
            return HttpResponseForbidden(
                force_text(_('You do not have permission to add a plugin.'))
            )

        pl_admin = target_placeholder._get_attached_admin()

        if pl_admin:
            template = pl_admin.get_placeholder_template(request, target_placeholder)
        else:
            template = None

        try:
            has_reached_plugin_limit(
                target_placeholder,
                module_plugin.plugin_type,
                language=language,
                template=template,
            )
        except PluginLimitReached as er:
            return HttpResponseBadRequest(er)

        tree_order = target_placeholder.get_plugin_tree_order(
            language=language,
            parent_id=target_plugin,
        )

        m_admin = module_plugin.placeholder._get_attached_admin()

        # This is needed only because we of the operation signal requiring
        # a version of the plugin that's not been committed to the db yet.
        new_module_plugin = copy.copy(module_plugin)
        new_module_plugin.pk = None
        new_module_plugin.placeholder = target_placeholder
        new_module_plugin.parent = None
        new_module_plugin.position = len(tree_order) + 1

        operation_token = m_admin._send_pre_placeholder_operation(
            request=request,
            placeholder=target_placeholder,
            tree_order=tree_order,
            operation=operations.ADD_PLUGIN,
            plugin=new_module_plugin,
        )

        new_plugins = copy_plugins_to_placeholder(
            plugins=list(module_plugin.get_unbound_plugins()),
            placeholder=target_placeholder,
            language=language,
            root_plugin=target_plugin,
        )
        tree_order.append(new_plugins[0].pk)
        reorder_plugins(
            target_placeholder,
            parent_id=target_plugin,
            language=language,
            order=tree_order,
        )
        new_module_plugin = cls.model.objects.get(pk=new_plugins[0].pk)

        m_admin._send_post_placeholder_operation(
            request,
            operation=operations.ADD_PLUGIN,
            token=operation_token,
            plugin=new_module_plugin,
            placeholder=new_module_plugin.placeholder,
            tree_order=tree_order,
        )

        response = cls().render_close_frame(request, obj=new_module_plugin)

        if form.cleaned_data.get('disable_future_confirmation'):
            response.set_cookie(key=cls.confirmation_cookie_name, value=True)
        return response

    @classmethod
    def modules_list_view(cls, request):
        if not request.user.is_staff:
            raise PermissionDenied

        view = ListView.as_view(
            model=Category,
            context_object_name='categories',
            queryset=Category.objects.order_by('name'),
            template_name='djangocms_modules/modules_list.html',
        )
        return view(request)


plugin_pool.register_plugin(Module)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class ModulesConfig(AppConfig):
    name = 'djangocms_modules'
    verbose_name = _('django CMS Modules')

    def ready(self):
        import djangocms_modules.handlers  # noqa



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models

import cms.models.fields

import djangocms_modules.models


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(unique=True, max_length=120, verbose_name='name')),
            ],
            options={
                'verbose_name': 'category',
                'verbose_name_plural': 'categories',
            },
        ),
        migrations.CreateModel(
            name='ModulePlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(parent_link=True, related_name='djangocms_modules_moduleplugin', auto_created=True, primary_key=True, serialize=False, to='cms.CMSPlugin', on_delete=models.CASCADE)),
                ('module_name', models.CharField(max_length=120, verbose_name='name')),
                ('module_category', models.ForeignKey(verbose_name='category', to='djangocms_modules.Category', on_delete=models.CASCADE)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='ModulesPlaceholder',
            fields=[
            ],
            options={
                'proxy': True,
            },
            bases=('cms.placeholder',),
        ),
        migrations.AddField(
            model_name='category',
            name='modules',
            field=cms.models.fields.PlaceholderField(slotname=djangocms_modules.models._get_placeholder_slot, editable=False, to='cms.Placeholder', null=True),
        ),
    ]



# -*- coding: utf-8 -*-
from django import forms
from django.conf import settings
from django.contrib import admin
from django.contrib.admin.widgets import (
    AdminTextInputWidget, RelatedFieldWidgetWrapper,
)
from django.utils.translation import ugettext_lazy as _

from cms.models import CMSPlugin, Placeholder

from .models import Category, ModulePlugin


class NewModuleForm(forms.Form):
    plugin = forms.ModelChoiceField(
        CMSPlugin.objects.exclude(plugin_type='Module'),
        required=False,
        widget=forms.HiddenInput(),
    )
    placeholder = forms.ModelChoiceField(
        queryset=Placeholder.objects.all(),
        required=False,
        widget=forms.HiddenInput(),
    )
    language = forms.ChoiceField(
        choices=settings.LANGUAGES,
        required=True,
        widget=forms.HiddenInput(),
    )

    def clean(self):
        if self.errors:
            return self.cleaned_data

        plugin = self.cleaned_data.get('plugin')
        placeholder = self.cleaned_data.get('placeholder')

        if not plugin and not placeholder:
            message = 'A plugin or placeholder is required to create a module'
            raise forms.ValidationError(message)

        if plugin and placeholder:
            message = 'A module can only be created from a plugin or placeholder, not both.'
            raise forms.ValidationError(message)
        return self.cleaned_data


class CreateModuleForm(NewModuleForm):

    name = forms.CharField(required=True, widget=AdminTextInputWidget)
    category = forms.ModelChoiceField(
        queryset=Category.objects.all(),
        required=True,
    )

    def set_category_widget(self, request):
        related_modeladmin = admin.site._registry.get(Category)
        dbfield = ModulePlugin._meta.get_field('module_category')
        formfield = self.fields['category']
        formfield.widget = RelatedFieldWidgetWrapper(
            formfield.widget,
            dbfield.rel if hasattr(dbfield, 'rel') else dbfield.remote_field,
            admin_site=admin.site,
            can_add_related=related_modeladmin.has_add_permission(request),
            can_change_related=related_modeladmin.has_change_permission(request),
            can_delete_related=related_modeladmin.has_delete_permission(request),
        )

    def get_plugins(self):
        plugin = self.cleaned_data.get('plugin')
        placeholder = self.cleaned_data.get('placeholder')

        if placeholder:
            language = self.cleaned_data['language']
            plugins = placeholder.get_plugins(language)
        else:
            plugins = plugin.get_tree(plugin).order_by('path')
        return list(plugins)


class AddModuleForm(forms.Form):
    target_plugin = forms.ModelChoiceField(
        CMSPlugin.objects.all(),
        required=False,
        widget=forms.HiddenInput(),
    )
    target_language = forms.ChoiceField(
        choices=settings.LANGUAGES,
        required=True,
        widget=forms.HiddenInput(),
    )
    target_placeholder = forms.ModelChoiceField(
        queryset=Placeholder.objects.all(),
        required=False,
        widget=forms.HiddenInput(),
    )
    disable_future_confirmation = forms.BooleanField(required=False, initial=False)

    def clean(self):
        if self.errors:
            return self.cleaned_data

        plugin = self.cleaned_data.get('target_plugin')
        placeholder = self.cleaned_data.get('target_placeholder')

        if not plugin and not placeholder:
            message = _('A plugin or placeholder is required to apply a module.')
            raise forms.ValidationError(message)

        if plugin and placeholder:
            message = _('A module can only be applied to a plugin or placeholder, not both.')
            raise forms.ValidationError(message)
        return self.cleaned_data



# -*- coding: utf-8 -*-
from django.contrib import admin

from cms.admin.placeholderadmin import PlaceholderAdminMixin

from .models import Category


class CategoryAdmin(PlaceholderAdminMixin, admin.ModelAdmin):
    list_display = ['name']


admin.site.register(Category, CategoryAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.db import models
from django.dispatch import receiver
from django.urls import Resolver404, resolve
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _

from cms import operations
from cms.models import CMSPlugin, Placeholder
from cms.models.fields import PlaceholderField
from cms.signals import pre_placeholder_operation
from cms.utils.plugins import get_bound_plugins

from six import python_2_unicode_compatible


def _get_placeholder_slot(category):
    return 'module-category-{}'.format(category.pk)


@receiver(pre_placeholder_operation)
def sync_module_plugin(sender, **kwargs):
    """
    Updates the created placeholder operation record,
    based on the configured post operation handlers.
    """
    operation_type = kwargs.pop('operation')
    affected_operations = (operations.MOVE_PLUGIN, operations.PASTE_PLUGIN)

    if operation_type not in affected_operations:
        return

    try:
        match = resolve(kwargs['origin'])
    except Resolver404:
        match = None

    is_in_modules = match and match.url_name == 'cms_modules_list'

    if not is_in_modules:
        return

    plugin = kwargs['plugin']
    placeholder = kwargs.get('target_placeholder')
    needs_sync = (
        plugin.plugin_type
        == 'Module'
        and placeholder.pk
        != plugin.module_category.modules_id
    )

    if needs_sync:
        # User has moved module to another category placeholder
        # or pasted a copied module plugin.
        new_category = Category.objects.get(modules=placeholder)
        (ModulePlugin
         .objects
         .filter(path__startswith=plugin.path, depth__gte=plugin.depth)
         .update(module_category=new_category))


@python_2_unicode_compatible
class Category(models.Model):
    name = models.CharField(
        verbose_name=_('Name'),
        max_length=120,
        unique=True,
    )
    modules = PlaceholderField(slotname=_get_placeholder_slot)

    class Meta:
        verbose_name = _('Category')
        verbose_name_plural = _('Categories')

    def __str__(self):
        return self.name

    @cached_property
    def modules_placeholder(self):
        return ModulesPlaceholder.objects.get(pk=self.modules_id)

    def get_non_empty_modules(self):
        unbound_plugins = (
            self
            .modules
            .get_plugins(language=settings.LANGUAGE_CODE)
            .filter(parent__isnull=True, numchild__gte=1)
        )
        return get_bound_plugins(unbound_plugins)


class ModulesPlaceholder(Placeholder):

    class Meta:
        proxy = True

    def _get_attached_model(self):
        return Category

    def _get_attached_models(self):
        return self._get_attached_model()

    def _get_attached_objects(self):
        return self._get_attached_model().objects.filter(modules=self.pk)

    @cached_property
    def category(self):
        return self._get_attached_model().objects.get(modules=self.pk)

    def get_label(self):
        return self.category.name


@python_2_unicode_compatible
class ModulePlugin(CMSPlugin):
    module_name = models.CharField(
        verbose_name=_('Name'),
        max_length=120,
    )
    module_category = models.ForeignKey(
        to=Category,
        verbose_name=_('Category'),
        on_delete=models.CASCADE,
    )

    def __str__(self):
        return self.module_name

    def update(self, refresh=False, **fields):
        ModulePlugin.objects.filter(pk=self.pk).update(**fields)
        if refresh:
            return self.reload()
        return

    def get_unbound_plugins(self):
        return CMSPlugin.get_tree(self).order_by('path')



# -*- coding: utf-8 -*-
from django import template
from django.conf import settings

from cms.utils.urlutils import admin_reverse

from ..models import Category


register = template.Library()


@register.simple_tag(takes_context=False)
def get_default_language():
    # le sigh...
    return settings.LANGUAGE_CODE


@register.simple_tag(takes_context=False)
def get_module_categories():
    return Category.objects.order_by('name')


@register.simple_tag()
def get_module_add_url(module_):
    return admin_reverse('cms_add_module', args=[module_.pk])


@register.simple_tag(takes_context=False)
def get_module_url(module_):
    return admin_reverse('cms_modules_list') + '#cms-plugin-{}'.format(module_.pk)
